/*
 *
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: Zhenyu Wang <wangzhenyu@kylinos.cn>
 *
 */

#include <QCoreApplication>
#include <QDebug>
#include "../src/kthememanager.h"

int main(int argc, char *argv[])
{
    QCoreApplication a(argc, argv);

    kdk::KThemeManager manager;

    QObject::connect(&manager, &kdk::KThemeManager::changed, [=](const QString property){
        qDebug() << property << "changed";
    });

    qDebug() << "UKCC Global Theme: " << manager.getUkccGlobalTheme();

    qDebug() << "Support style: " << manager.getSupportSystemStyle();

    qDebug() << "System font: " << manager.getSystemFont();

    qDebug() << "System font size: " << manager.getSystemFontSize();

    qDebug() << "System color: " << manager.getSystemThemeColor();

    qDebug() << "Set system style: " << manager.setSystemStyle("ukui-default");

    qDebug() << "Set system font: " << manager.setSystemFont("Noto Sans CJK SC");

    qDebug() << "Set system font size: " << manager.setSystemFontSize(12);

    qDebug() << "Set theme color: " << manager.setSystemThemeColor("daybreakBlue");

    qDebug() << "Set panel special effect: " << manager.setPanelSpecialEffect(true);

    qDebug() << "Set transparency: " << manager.setTransparency(0.70);

    return a.exec();
}
